package com.zanxiang.game.module.sdk.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.module.mybatis.entity.Agent;
import com.zanxiang.game.module.mybatis.entity.User;
import com.zanxiang.game.module.mybatis.entity.UserLoginLog;
import com.zanxiang.game.module.mybatis.mapper.AgentMapper;
import com.zanxiang.game.module.sdk.service.IAgentService;
import com.zanxiang.game.module.sdk.service.ICallBackService;
import com.zanxiang.game.module.sdk.service.IUserLoginLogService;
import com.zanxiang.game.module.sdk.service.IUserService;
import com.zanxiang.module.util.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.util.function.Tuple3;
import reactor.util.function.Tuples;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author bilingfeng
 */
@Slf4j
@Service
public class AgentServiceImpl extends ServiceImpl<AgentMapper, Agent> implements IAgentService {

    @Autowired
    private IUserService userService;

    @Autowired
    private IUserLoginLogService userLoginLogService;

    @Autowired
    private ICallBackService callBackService;

    @Override
    public void userAgentUpdate(User user, String channel) {
        log.error("用户登录接收到的渠道参数 userId : {}, channel : {}", user.getId(), channel);
        //渠道参数为空
        if (Strings.isBlank(channel)) {
            return;
        }
        //判断用户是否超过30天未登录
        List<UserLoginLog> userLoginLogList = userLoginLogService.list(new LambdaQueryWrapper<UserLoginLog>()
                .eq(UserLoginLog::getUserId, user.getId())
                .orderByDesc(UserLoginLog::getCreateTime)
                .last("limit 1"));
        if (CollectionUtils.isEmpty(userLoginLogList)) {
            return;
        }
        //最近登录时间
        LocalDateTime lastLoginTime = userLoginLogList.get(0).getCreateTime();
        //最近登录未超过30天
        if (lastLoginTime.plusDays(30L).isAfter(LocalDateTime.now())) {
            return;
        }
        //30天未登录, 更新渠道
        Map<String, String> urlParamMap = this.channelTransform(channel);
        //查询渠道
        Agent agent = this.getAgentByKey(urlParamMap);
        if (agent == null) {
            return;
        }
        //更新用户信息
        user.setAgentId(agent.getId());
        user.setChannel(channel);
        user.setUpdateTime(LocalDateTime.now());
        userService.updateById(user);
        //回传用户信息
        callBackService.userCallBack(user, urlParamMap);
    }

    @Override
    public Tuple3<Long, Map<String, String>, String> getUserAgentId(String channel) {
        log.error("用户注册接收到的渠道参数 channel : {}", channel);
        if (Strings.isBlank(channel)) {
            return Tuples.of(0L, Collections.emptyMap(), Strings.EMPTY);
        }
        //数据解析
        Map<String, String> urlParamMap = this.channelTransform(channel);
        String shareUserId = urlParamMap.get("shareUserId");
        //查询渠道
        Agent agent = null;
        if (Strings.isNotBlank(shareUserId)) {
            User user = userService.getById(Long.valueOf(shareUserId));
            if (user != null) {
                agent = super.getById(user.getAgentId());
            }
        } else {
            agent = this.getAgentByKey(urlParamMap);
        }
        if (agent == null) {
            return Tuples.of(0L, Collections.emptyMap(), Strings.isBlank(shareUserId) ? Strings.EMPTY : shareUserId);
        }
        return Tuples.of(agent.getId(), urlParamMap, Strings.isBlank(shareUserId) ? Strings.EMPTY : shareUserId);
    }

    private Agent getAgentByKey(Map<String, String> urlParamMap) {
        //参数不存在
        if (CollectionUtils.isEmpty(urlParamMap)) {
            return null;
        }
        //判断渠道标识
        String state = urlParamMap.get("agentKey");
        if (Strings.isBlank(state)) {
            state = urlParamMap.get("state");
        }
        return super.getOne(new LambdaQueryWrapper<Agent>().eq(Agent::getAgentKey, state));
    }

    @Override
    public Map<String, String> channelTransform(String channel) {
        //判断是否是json, 是就直接转成map
        if (this.isJsonStr(channel)) {
            return JsonUtil.toMap(channel, Map.class, String.class);
        }
        //非json, 自己解析
        Map<String, String> paramMap = new HashMap<>(11);
        String[] params = channel.split("&");
        for (String param : params) {
            String[] keyValue = param.split("=");
            if (keyValue.length > 1) {
                String key = keyValue[0];
                String value = keyValue[1];
                paramMap.put(key, value);
            }
        }
        return paramMap;
    }

    private boolean isJsonStr(String str) {
        boolean result = false;
        try {
            JSON.parse(str);
            result = true;
        } catch (Exception ignored) {
        }
        return result;
    }
}
